function planet(massa_tgt,x_tgt,y_tgt,z_tgt,vx_tgt,vy_tgt,vz_tgt,...
    massa_int,x_int,y_int,z_int,vx_int,vy_int,vz_int,...
    dT_iter,iteraz, dT);
% PLANET Simulazione dell'interazione tra pianeti

% Inizializzazione del campo grafico di disegno
f=figure;
set(f,'Renderer','Zbuffer',...
    'Color',[.6 .6 .6]);
axh=axes('Box','on',...
    'DrawMode','Fast',...
    'DataAspectRatio',[1, 1, 1],...
    'Position',[0.1 0.1 0.8 0.8],...
    'Color',[0 0 0],...
    'XColor',[1 1 1],...
    'YColor',[1 1 1],...
    'ZColor',[1 1 1]);
view([-45 45]);
axis equal
xlabel('asse X ()');
ylabel('asse Y ()');
zlabel('asse Z ()');

% Inizializzazione variabili
G = 6.67259e-11; % N * m^2 / kg^2

hold on

% Centri del TARGET e dell'INTRUDER
bersaglio = plot3(x_tgt,y_tgt,z_tgt,'Marker','.','MarkerSize',25,'Color',[1 1 1]);
intruso = plot3(x_int,y_int,z_int,'Marker','.','MarkerSize',25,'Color',[1 1 1]);

drawnow
axis equal;

% In prima approssimazione possiamo dire che per calcolare la posizione
% di un pianeta che ha posizione iniziale (x0, y0, z0):
% x1=x0+u0*dT
% y1=y0+v0*dT
% z1=z0+w0*dT
% Dato il calcolo delle velocita' a partire dalle forze di interazione
% scambiate tra i corpi e le velocita' iniziali (u0, v0, w0):
% u1=u0+Fx*dT
% v1=v0+Fy*dT
% w1=w0+Fz*dT

for j=1:iteraz
    for k=1:dT_iter
        % Calcolo della velocita' del centro del pianeta TARGET
        opt1 = x_tgt - x_int;
        opt2 = y_tgt - y_int;
        opt3 = z_tgt - z_int;
        opt4 = (opt1^2 + opt2^2 + opt3^2 + 1e9)^1.5;
        vx_tgt = vx_tgt - G*massa_int*opt1/opt4*dT;
        vy_tgt = vy_tgt - G*massa_int*opt2/opt4*dT;
        vz_tgt = vz_tgt - G*massa_int*opt3/opt4*dT;
        % Calcolo della velocita' del centro del pianeta INTRUDER
        vx_int = vx_int + G*massa_tgt*opt1/opt4*dT;
        vy_int = vy_int + G*massa_tgt*opt2/opt4*dT;
        vz_int = vz_int + G*massa_tgt*opt3/opt4*dT;
        % Aggiornamento della posizione del centro del pianeta TARGET
        x_tgt = x_tgt+vx_tgt*dT;
        y_tgt = y_tgt+vy_tgt*dT;
        z_tgt = z_tgt+vz_tgt*dT;
        % Aggiornamento della posizione del centro del pianeta INTRUDER
        x_int = x_int+vx_int*dT;
        y_int = y_int+vy_int*dT;
        z_int = z_int+vz_int*dT;
        % Disegno della traiettoria dei centri dei pianeti
        line([x_tgt-vx_tgt*dT x_tgt],[y_tgt-vy_tgt*dT y_tgt],[z_tgt-vz_tgt*dT z_tgt],'Color',[1 1 0],'linewidth',2);
        line([x_int-vx_int*dT x_int],[y_int-vy_int*dT y_int],[z_int-vz_int*dT z_int],'Color',[1 1 0],'linewidth',2);
    end
    % Varizione della posizione dei centri dei pianeti
    set(bersaglio,'XData',x_tgt,'YData',y_tgt,'ZData',z_tgt);
    set(intruso,'XData',x_int,'YData',y_int,'ZData',z_int);
    % Calcolo del tempo trascorso
    tempo=(j-1)*dT_iter+k;
    % Aggiornamento delle visualizzazione
    title(['Tempo: ',num2str(tempo),' /',num2str(iteraz*dT_iter)]);
    drawnow
    axis equal;
end