/***************************************************************************
                             ppc.cc  -  Preprocessore per crobots
                             ------------------------------------
    begin                : thu jun 6 11:00:36 CET 2000
    copyright            : (C) 2000 by Marco Giovannini
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
//#include <unistd.h>
#include <ctype.h>
#include "h/ppc.h"
#include "h/deflist.h"
#include "h/token.h"
#include "h/preproc.h"

PPCError error;
DefList  def_list;

FILE* fopen_read(char* fname, PPCError* error)
{
    FILE* tmp=fopen(fname,"r");
    if (!tmp) error->set(PPC_FILENOTEXIST,NULL,0);
    return tmp;
}

FILE* fopen_write(char* fname, PPCError* error)
{
    FILE* tmp=fopen(fname,"w");
    if (!tmp) error->set(PPC_FILENOTEXIST,NULL,0);
    return tmp;
}

static int PPC(char* in, char* out, PPCError* error, DefList* def_list)
{
    int err;
    FILE* fout;
    if (!(fout=fopen_write(out,error))) return error->getCode();
    PreProcessor* ppc=new PreProcessor(fout,error,def_list);
    if (err=ppc->start(in)) error->setFileName(in);
    fclose(fout);
    return err;
}

static void intro(void)
{
  int i;
  char version[20]="1.0 RC";

  printf("CRobots Preprocessor  Versione %s\n",version);
  printf(" (C)  Marco  Giovannini   2000\n");
  printf(" [Modified  By Simone Ascheri]\n\n");
}

void usage(void)
{
  fprintf(stderr,"Uso: ppc file.(p)r [file.(p)r [file.(p)r [file.(p)r]]] [-mxxx -lxxxx]\n\n");
  fprintf(stderr,"Esempi:\n");
  fprintf(stderr,"=======\n");
  fprintf(stderr," PPC aeris.pr             [Pre-processa il file aeris.pr]\n");
  fprintf(stderr,"                          [Scrive il risultato in aeris.r]\n\n");
  fprintf(stderr," PPC aeris.pr goblin.r shock.pr jedi.r\n");
  fprintf(stderr,"                          [Pre-processa i file aeris.pr e shock.pr]\n");
  fprintf(stderr,"                          [Lancia crobots con i 4 robot e gli switch]\n");
  fprintf(stderr,"                           passati sulla linea di comando]\n");
  fprintf(stderr,"                          [Cancella aeris.r e shock.r]\n");
  fprintf(stderr,"Switch:\n");
  fprintf(stderr,"=======\n");
  fprintf(stderr," -mxxx                    Identico a Crobots\n");
  fprintf(stderr," -lxxx                    Identico a Crobots\n");
  fprintf(stderr," -\">xxx\"                  Corrisponde a >xxx in Crobots\n");
  fprintf(stderr," -\"<xxx\"                  Corrisponde a <xxx in Crobots\n");
  fprintf(stderr," -v=varname:xxx");
  fprintf(stderr,"           [varname:xxx corrisponde a #define varname xxx]\n");
  fprintf(stderr,"                          [all'interno del file .pr]\n");
  exit(-1);
}

static int quit(PPCError* error)
{
    char* str;
    switch (error->getCode()) {
        case PPC_FILENOTEXIST : str="il file non esiste"; break;
        case PPC_NODEFNAME    : str="il nome della definizione non e' presente"; break;
        case PPC_DEFNOTDEFINED: str="la definizione non esiste"; break;
        case PPC_NOTINIF      : str="espressione if inaspettata"; break;
        case PPC_NODEFVALUE   : str="il valore della definizione non e' presente"; break;
    }
    fprintf(stderr,"PPC: errore (%d) nel file \"%s\" (linea %d): %s!\n",
            error->getCode(),error->getFileName(),error->getLine(),str);
    exit(error->getCode());
}

inline void my_strcat(char* dst, char* src)
{
    strcat(dst,src);
    strcat(dst," ");
}

int main(int argc, char *argv[])
{
    int  n_robr=0, n_robpr=0, n_params=0;
    char robr[4][80], robpr[4][80], params[MAXPARAMS][80];
    int i;

    intro();
    if (argc<2) usage();

    //aggiunge alla lista delle definizioni quelle passate su linea di
    //comando
    for (i=1; i<argc; i++) {
        if(argv[i][0]=='-' && tolower(argv[i][1])=='v') {
            char def_name[100], def_value[100];
            strcpy(def_name,strtok(argv[i]+3,":"));
            strcpy(def_value,argv[i]+strlen(def_name)+4);
            Definition* def=new Definition(def_name,def_value);
            def_list.add(def);
        }
    }

    //passa in rassegna i vari prametri, aggiornando l'array corrispondente:
    //robot da processare, robot processato, parametro per crobots
    for (i=1; i<argc; i++) {
        if (strstr(argv[i],".pr")) {
            robpr[n_robpr][0]='\0';
            strncat(robpr[n_robpr],argv[i],strlen(argv[i])-3);
            strcat(robpr[n_robpr],".r");
	        if (PPC(argv[i],robpr[n_robpr],&error,&def_list)!=PPC_OK)
                return quit(&error);
            n_robpr++;
        } else if (strstr(argv[i],".r")) {
            strcpy(robr[n_robr++],argv[i]);
        } else if (argv[i][0]=='-' && argv[i][1]!='v') {
            if (argv[i][1]=='<' || argv[i][1]=='>') {
                strcpy(params[n_params++],argv[i]+1);
            } else {
                strcpy(params[n_params++],argv[i]);
            }
        } else strcpy(params[n_params++],argv[i]);
    }

    //se sono stati passati piu' robot su linea di comando, e' perche'
    //dobbiamo anche eseguire crobots
    if (n_robr+n_robpr>1) {
        char tmp[200]="crobots ";
        for (i=0; i<n_robr; i++) my_strcat(tmp,robr[i]);
        for (i=0; i<n_robpr; i++) my_strcat(tmp,robpr[i]);
        for (i=0; i<n_params; i++) my_strcat(tmp,params[i]);
        printf("%s\n",tmp);
        system(tmp);

        //cancella i file .r creati dai .pr
        if (n_robpr>0) {
#ifdef __MSDOS__
            strcpy(tmp,"del ");
#else
            strcpy(tmp,"rm -f ");
#endif
            for (int i=0; i<n_robpr; i++) my_strcat(tmp,robpr[i]);
            system(tmp);
        }
    }
    return PPC_OK;
}

