/***************************************************************************
                             preproc.cc  -  Preprocessore C
                             -----------------------------------
    begin                : thu jun 6 11:00:36 CET 2000
    copyright            : (C) 2000 by Marco Giovannini
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include "h/ppc.h"
#include "h/preproc.h"

PreProcessor::PreProcessor(FILE* dst_file, PPCError* err, DefList* defs)
{
    in_if=in_cmt=FALSE;
    if_flag=TRUE;
    last='\0';
    last_def=NULL;
    dst=dst_file;
    if (defs) dl=defs;
    else dl=new DefList;
    error=err;
}

int PreProcessor::start(char* in)
{
    FILE* fin;
    if (!(fin=fopen_read(in,error))) return error->getCode();
    if (processFile(fin)) {
        if (!(error->getFileName())) error->setFileName(in);
        return error->getCode();
    }
    fclose(fin);
    return PPC_OK;
}

int PreProcessor::processFile(FILE* src_file)
{
    char buf[BUFSIZE];
    int err=0, line=0;
    while (fgets(buf,READBUFSIZE,src_file) && !err) {
        tok.tokenize(buf);
        if (err=processLine(++line)) {
            if (!error->getCode()) error->setCode(err);
            if (!error->getLine()) error->setLine(line);
        }
    }
    return err;
}

int PreProcessor::processLine(int line)
{
    int err=PPC_OK;
    if (char* tmp=tok.trim()) {
        if (!in_cmt && *tmp=='#') err=processDirective(tmp);
        else if (if_flag) {
            if (last_def) continueDefine(last_def,tmp);
            else {
                tok.firstToken();
                tmp=tok.getToken();
                while (tmp && err==PPC_OK) {
                    err=processWord(tmp);
                    tmp=tok.nextToken();
                }
                tok.printAll(dst);
            }
        }
    } else tok.printAll(dst);
    return err;
}

int PreProcessor::processDirective(char* dir)
{
    int err=PPC_OK;
    if (!strcmp(dir,"#else")) err=processElse();
    else if (!strcmp(dir,"#endif")) err=processEndif();
    else if (if_flag) {
        if (!strcmp(dir,"#define")) err=processDefine();
        else if (!strcmp(dir,"#ifdef")) err=processIfdef();
        else if (!strcmp(dir,"#ifndef")) err=processIfndef();
        else if (!strcmp(dir,"#include")) err=processInclude();
        else if (!strcmp(dir,"#undef")) err=processUndef();
    }
    return err;
}

int PreProcessor::processWord(char* word)
{
    char tmp_last=*word;
    if (last=='*' && *word=='/') {
        in_cmt=FALSE;
        tmp_last='\0';
    } else if (!in_cmt) {
        if (*word=='/') {
            if (last=='/') {
                *word='*';
                tok.lastToken();               //l'ultimo token e' "/n"
                tok.changeToken("*/\n");
                tmp_last='\0';
            }
        } else if (*word=='*') {
            if (last=='/') {
                in_cmt=TRUE;
                tmp_last='\0';
            }
        } else if (*word=='$') {
             last='\0';
             return processDefinition(++word); 
        }
    }
    last=tmp_last;
    return PPC_OK;
}

int PreProcessor::processDefinition(char* def_name)
{
    if (Definition* def=dl->search(def_name)) {
        tok.changeToken(def->getValue());
    } else return PPC_DEFNOTDEFINED;
    return PPC_OK;
}

#define GETNAME(name) if (!(name=tok.trimNext())) return PPC_NODEFNAME

void PreProcessor::continueDefine(Definition* def, char* value)
{
    char *val=value;
    if (*val!='\\') {
        def->catValue(value);
        if (val=tok.nextToken()) {
            while (val && *val!='\n' && *val!='\\') {
                def->catValue(val);
                val=tok.nextToken();
            }
        }
    }
    if (*val=='\\') last_def=def;
    else last_def=NULL;
}

int PreProcessor::processDefine(void)
{
    char* name;
    GETNAME(name);
    Definition* def=new Definition(name);
    if (char* val=tok.trimNext()) {
        if (*val=='(') {
            do {
               if ((val=tok.trimNext()) && *val!=')' && *val!=',') {
                    Definition* param=new Definition(val);
                    def->addParam(param);
                }
            } while (val && *val!=')');
            if (!val) return PPC_NODEFVALUE;
            val=tok.trimNext();
        }
        if (val) {
            def->setValue("");
            continueDefine(def,val);
        }
    }
    dl->add(def);
    return PPC_OK;
}

int PreProcessor::processUndef(void)
{
    char* name;
    GETNAME(name);
    if (Definition* def=dl->search(name)) dl->remove(def);
    return PPC_OK;
}

int PreProcessor::processIfdef(void)
{
    char* name;
    GETNAME(name);
    if (!(dl->search(name))) if_flag=FALSE;
    in_if=TRUE;
    return PPC_OK;
}

int PreProcessor::processIfndef(void)
{
    char* name;
    GETNAME(name);
    if (dl->search(name)) if_flag=FALSE;
    in_if=TRUE;
    return PPC_OK;
}

int PreProcessor::processElse(void)
{
    if (!in_if) return PPC_NOTINIF;
    if_flag=!if_flag;
    return PPC_OK;
}

int PreProcessor::processEndif(void)
{
    if (!in_if) return PPC_NOTINIF;
    if_flag=TRUE;
    in_if=FALSE;
    return PPC_OK;
}

int PreProcessor::processInclude(void)
{
    int err;
    char* in;
    char* end_in;
    GETNAME(in);
    if (*in=='\"' || *in=='<') ++in;
    end_in=in+strlen(in)-1;
    if (*end_in=='\"' || *end_in=='>') *end_in='\0';
    if (err=start(in)) error->setFileName(in);
    return err;
}

