package it.edmaster.eLisa;

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import it.edmaster.eLisa.carrello.*;
import it.edmaster.eLisa.catalogo.*;
import it.edmaster.eLisa.ordini.*;
import it.edmaster.eLisa.utenti.*;

/**
 * Questa classe si occupa di reindirizzare le richieste alla servlet controller alla risorsa
 * web corretta.<P>
 *
 * @see it.edmaster.eLisa.ControllerServlet
 */
class DispatcherHelper {
    
    /**
     * Verifica l'operazione richiesta e richiama la risorsa web collegata
     *
     * @param op operazione da effettuare (p.e. catalogo.dettaglioProdotto)
     * @param req richiesta alla servlet associata
     * @param res risposta della servlet associata
     */
    void dispatch( String op, HttpServletRequest req, HttpServletResponse res) throws IOException, 
            ServletException {
        
        if( op == null ) {
            forward( "/include/nonTrovata.jsp" , req, res);
        }
        
        //Azione predefinita
        if( op.equals("home") ) {
            op = "catalogo.mostra";
        }
        
        if( op.startsWith("catalogo") ) {
            catalogo( op, req, res );
        } else if( op.startsWith( "carrello" ) ) {
            carrello( op, req, res );
        } else if( op.startsWith( "utenti" ) ) {
            utenti( op, req, res );
        } else if( op.startsWith( "ordini" ) ) {
            ordini( op, req, res );
        }

        forward( "/include/NonTrovata.jsp", req, res );
    }
    
    private void catalogo( String op, HttpServletRequest req, HttpServletResponse res) throws IOException, 
            ServletException {
                
        if( op.equals( "catalogo.dettagliProdotto" ) ) {
            forward( "/catalogo/Prodotto.jsp", req, res );
        } else if( op.equals( "catalogo.mostra" ) ) {
            forward( "/catalogo/Catalogo.jsp", req, res );
        } else if( op.equals( "catalogo.cerca" ) ) {
            forward( "/catalogo/Catalogo.jsp", req, res );
        } else if( op.equals( "catalogo.elencaPerCategoria" ) ) {
            forward( "/catalogo/Catalogo.jsp", req, res );
        }
    }
    
    private void carrello( String op, HttpServletRequest req, HttpServletResponse res) throws IOException, 
            ServletException {

        CatalogoHelper catalogo = null;
        
        CarrelloHelper ch = (CarrelloHelper)req.getSession().getAttribute("carrello");
        if( ch == null ) {
            catalogo = (CatalogoHelper)req.getSession().getAttribute("catalogo");
            if( catalogo == null ) {
                throw new ServletException("Errore interno: non trovo il catalogo prodotti");
            }
            ch = new CarrelloHelper( catalogo );
            req.getSession().setAttribute("carrello", ch );
        }
                
        if( op.equals( "carrello.aggiungi" ) ) {
            try {
                ch.aggiungi( req.getParameter("id") );
            } catch( CaricamentoProdottiException ex ) {
                throw new ServletException( ex );
            }
            
            //Visualizza di nuovo il prodotto selezionato
            catalogo( "catalogo.dettagliProdotto", req, res );
        } else if( op.equals( "carrello.mostra" ) ) {
            forward( "/carrello/Carrello.jsp", req, res );
        } else if( op.equals( "carrello.svuota" ) ) {
            ch.svuota();
            forward( "/carrello/Carrello.jsp", req, res );
        } else if( op.equals( "carrello.checkout.1" ) ) {
            forward( "/carrello/Conferma.jsp", req, res );
        } else if( op.equals( "carrello.checkout.2" ) ) {
            Utente utente = (Utente)req.getSession().getAttribute("utenteConnesso");
            if( utente != null ) {
                String codiceUtente = utente.getCodiceUtente();
                try {
                    ch.inoltraOrdine(codiceUtente);
                } catch( InserimentoOrdineException ex ) {
                    throw new ServletException( ex );
                }
                forward( "/carrello/Ringrazia.jsp", req, res );
            } else {
                //PENDING: errore interno
                forward( "/include/nonTrovata.jsp" , req, res);
            }
        }
    }
    
    private void utenti( String op, HttpServletRequest req, HttpServletResponse res) throws IOException, 
            ServletException {

        ElencoUtentiHelper eu = creaElencoUtenti( req, res );
        
        if( op.equals( "utenti.login" ) ) {
            try {
                if( eu.autentica( 
                        req.getParameter("utente"),
                        req.getParameter("password") ) ) {
                            
                    req.getSession().setAttribute("utenteConnesso", 
                        eu.getUtente(req.getParameter("utente")) 
                    );
                    
                    //chiama indirettamente per far eventualmente elaborare il metodo relativo
                    dispatch( "catalogo.mostra", req, res );
                } else {
                    forward( "/utenti/LoginErrato.jsp", req, res );
                }
            } catch( CaricamentoUtentiException ex ) {
                throw new ServletException( ex );
            }
        } else if( op.equals( "utenti.nuovo" ) ) {
            forward( "/utenti/Nuovo.jsp", req, res );
        } else if( op.equals( "utenti.nuovo.salva" ) ) {
            String sottoOp = req.getParameter("conferma");
            if( sottoOp != null && sottoOp.length()>0 ) {
                try {
                    eu.inserisci( req.getParameter("codiceUtente"),
                        req.getParameter("password1"), req.getParameter("password2"),
                        req.getParameter("nome"), req.getParameter("cognome") 
                    );
                    System.out.println("utenti.nuovo.salva " + eu.getUtente(req.getParameter("codiceUtente") ) );
                    req.getSession().setAttribute("utenteConnesso", 
                        eu.getUtente(req.getParameter("codiceUtente")) 
                    );
                    forward( "/utenti/UtenteInserito.jsp", req, res );
                } catch( CaricamentoUtentiException ex1 ) {
                    throw new ServletException( ex1 );
                } catch( InserimentoUtentiException ex2 ) {
                    req.setAttribute( "errore", ex2 );
                    forward( "/utenti/Nuovo.jsp", req, res );
                }
            }
            dispatch( "home", req, res );
        } else if( op.equals( "utenti.passwordPersa" ) ) {
            forward( "/utenti/PasswordPersa.jsp", req, res );
        } else if( op.equals( "utenti.passwordPersa.cerca" ) ) {
            String sottoOp = req.getParameter("conferma");
            if( sottoOp != null && sottoOp.length()>0 ) {
                try {
                    String password = eu.cercaPassword( req.getParameter("codiceUtente"),
                        req.getParameter("nome"), req.getParameter("cognome") 
                    );
                    if( password == null ) {
                        req.setAttribute( "errore", "I dati inseriti non risultano validi" );
                    } else {
                        req.setAttribute( "password", password );
                    }
                    forward( "/utenti/PasswordPersa.jsp", req, res );
                } catch( CaricamentoUtentiException ex1 ) {
                    throw new ServletException( ex1 );
                }
            }
            dispatch( "home", req, res );
        } else if( op.equals( "utenti.logout" ) ) {
            req.getSession().removeAttribute("utenteConnesso");
            dispatch( "home", req, res );
        }
    }
    
    private void ordini( String op, HttpServletRequest req, HttpServletResponse res) throws IOException, 
            ServletException {
        if( op.equals( "ordini.elenco" ) ) {
            forward( "/ordini/Ordini.jsp", req, res );
        } if( op.equals( "ordini.mostra" ) ) {
            forward( "/ordini/Ordine.jsp", req, res );
        }
    }
    
    private ElencoUtentiHelper creaElencoUtenti(HttpServletRequest req, HttpServletResponse res) {
        ElencoUtentiHelper eu = (ElencoUtentiHelper)req.getSession().getAttribute("utente");
        
        if( eu == null ) {
            eu = new ElencoUtentiHelper();
            req.getSession().setAttribute("utente", eu );
        }
        
        return eu;
    }
    
    /**
     * Esegue un forward standard alla risorsa indicata dal parametro <code>resourceURI</code>
     *
     * @param resourceURI risorsa da richiamare
     * @param req richiesta alla servlet associata
     * @param res risposta della servlet associata
     */
    void forward( String resourceURI, HttpServletRequest req, HttpServletResponse res) 
            throws IOException, ServletException {
                
        RequestDispatcher dispatcher = req.getRequestDispatcher( resourceURI );
        dispatcher.forward( req, res );
        
    }
}
