package it.edmaster.eLisa.catalogo;

import java.sql.*;
import java.util.*;

import it.edmaster.eLisa.*;
import it.edmaster.eLisa.common.*;
import it.edmaster.eLisa.util.*;

public class Catalogo {

    /**
     * @clientCardinality 1..*
     * @link aggregation 
     */
    private Prodotto lnkProdotto;
    private CatalogoDAO lnkCatalogoDAO;
    
    /**
     * Il catalogo prodotti definisce l'insieme di prodotti gestiti dal sistema.<BR>
     * E' possibile ottenere l'elenco completo dei prodotti in catalogo, o selezionare per
     * categoria. Gli elenchi vengono ritornati tramite oggetti <code>Pagina</code> 
     * che consentono lo scorrimento
     * a pagine all'interno della lista completa.
     * <P>
     * Questa classe consente anche l'accesso alle categorie di prodotti
     *
     * @see it.edmaster.eLisa.common.Pagina
     */
    public Catalogo() {
    }
    
    /** 
     * Ritorna le possibili categorie in cui possono essere inseriti i prodotti
     */
    public List getCategorieProdotto() throws CaricamentoCategorieException {
        List categoriaProdotto = null;
        
        CategoriaProdottoDAO dao = DAOLocator.getInstance().getCategoriaProdottoDAO();
        try {
            categoriaProdotto = dao.load();
        } catch( SQLException se ) {
            throw new CaricamentoCategorieException( se );
        }
        
        return categoriaProdotto;
    }
    
    public Prodotto getProdotto( String id ) throws CaricamentoProdottiException {
        Prodotto prodotto = null;
        
        CatalogoDAO dao = DAOLocator.getInstance().getCatalogoDAO();
        
        try {
            prodotto = dao.getProdotto( id );
        } catch( SQLException se ) {
            throw new CaricamentoProdottiException( se );
        }
        
        //Logger.log("Catalogo.getProdotto()="+prodotto);
        
        return prodotto;
    }
    
    /**
     * Ritorna una pagina dell'elenco dei prodotti a partire dal prodotto <code>inizio</code>
     * e per il numero di elementi specificato da <code>quanti</code>
     *
     * @param inizio numero di prodotto iniziale
     * @param quanti numero di prodotti da ritornare
     *
     * @return la pagina di dati richiesta
     * @throws CaricamentoProdottiException segnala eventuali errori di accesso ai dati
     * @see Pagina
     */
    public Pagina getElenco( int inizio, int quanti ) throws CaricamentoProdottiException {
        Pagina pagina = null;
        
        CatalogoDAO dao = DAOLocator.getInstance().getCatalogoDAO();
        
        try {
            pagina = dao.getElenco( inizio, quanti );
        } catch( SQLException se ) {
            throw new CaricamentoProdottiException( se );
        }
        
        //Logger.log( "getElenco() " + pagina );

        return pagina;
    }
    
    public Pagina getElenco( String idCategoria, int inizio, int quanti ) 
            throws CaricamentoProdottiException {
        Pagina pagina = null;
        
        CatalogoDAO dao = DAOLocator.getInstance().getCatalogoDAO();
        
        try {
            pagina = dao.getElenco( idCategoria, inizio, quanti );
        } catch( SQLException se ) {
            throw new CaricamentoProdottiException( se );
        }
        
        //Logger.log( "getElenco() " + pagina );

        return pagina;
    }
    
    /**
     *
     */
    public Pagina cerca( String ricerca, int inizio, int quanti ) throws DataAccessException {
        Pagina pagina = null;
        
        CatalogoDAO dao = DAOLocator.getInstance().getCatalogoDAO();
        
        try {
            pagina = dao.cerca( ricerca, inizio, quanti );
        } catch( SQLException se ) {
            throw new DataAccessException( se );
        }
        
        return pagina;
    }
    
}
