package it.edmaster.eLisa.catalogo;

import java.sql.*;
import java.util.*;

import it.edmaster.eLisa.*;
import it.edmaster.eLisa.common.*;
import it.edmaster.eLisa.util.*;

public class CatalogoDAO {
    ApplicationContext context;

    /** @link dependency */
    /*#CaricamentoProdottiException lnkCaricamentoProdottiException;*/
    
    public CatalogoDAO( ApplicationContext context ) {
        this.context = context;
    }
    
    public Prodotto getProdotto( String id ) throws SQLException {
        Prodotto prodotto = (Prodotto)CacheManager.getCacheProdotti().get( id );
        
        if( prodotto == null ) {
            prodotto = new Prodotto( id );
            CacheManager.getCacheProdotti().add( id, prodotto );
        }
        
        return prodotto;
    }
    
    public Pagina getElenco(int inizio, int quanti) throws SQLException {
        return loadElenco( null, null, inizio, quanti );
    }
    
    public Pagina getElenco( String idCategoria, int inizio, int quanti) throws SQLException {
        return loadElenco( idCategoria, null, inizio, quanti );
    }
    
    public Pagina cerca( String ricerca, int inizio, int quanti ) throws SQLException {
        return loadElenco( null, ricerca, inizio, quanti );
    }
    
    Pagina loadElenco( String idCategoria, String ricerca, int inizio, int quanti ) 
            throws SQLException {
        Pagina ritorno = null;
        String sql = "SELECT ID, NOME, DESCRIZIONE FROM PRODOTTI";
        
        if( ricerca != null ) {
            sql += " WHERE NOME LIKE '%" + ricerca + "%' OR " +
                   "DESCRIZIONE LIKE '%" + ricerca + "%'";
        }
        
        if( idCategoria != null ) {
            sql += " WHERE CATEGORIA=" + idCategoria;
        }
        
        sql += " ORDER BY ID";
        
        Connection conn = null;
        Statement stmt = null;
        ResultSet rs = null;
        
        try {
            conn = context.getConnection();
            stmt = conn.createStatement( 
                ResultSet.TYPE_SCROLL_INSENSITIVE,
                ResultSet.CONCUR_READ_ONLY
            );
            rs = stmt.executeQuery( Logger.logSql( sql ) );
            
            Logger.log( "CatalogoDAO.loadElenco(): inizio="+inizio + " absolute="+rs.absolute(inizio+1) );
            
            if (inizio >= 0 && rs.absolute(inizio+1)) {
                boolean hasNext = false;
                List elenco = new LinkedList();
                
                do {
                    int pos = 1;
                    
                    Prodotto cp = null;
                    String id = rs.getString( pos++ );
                    
                    if( !CacheManager.getCacheProdotti().contains( id ) ) {
                        String nome = rs.getString( pos++ );
                        String descrizione = rs.getString( pos++ );
                        
                        cp = new Prodotto( id, nome, descrizione );
                        
                        //Salva il prodotto nella cache
                        CacheManager.getCacheProdotti().add( id, cp );
                    } else {
                        cp = (Prodotto)CacheManager.getCacheProdotti().get( id );
                    }
                    elenco.add( cp );
                    
                    //Logger.log( "CatalogoDAO.loadElenco(): quanti="+quanti );
                    
                } while ((hasNext = rs.next()) && (--quanti > 0));
                
                ritorno = new Pagina(elenco, inizio, hasNext);
            } else {
                ritorno = Pagina.VUOTA;
            }
        
            rs.close();
            stmt.close();
        } catch( SQLException ex ) {
            throw ex;
        } finally {
            JDBCUtils.closeAll( conn, stmt, rs );
        }
        
        return ritorno;
    }
    
    /**
     * Carica in memoria tutte le informazioni relative ad un prodotto, a partire da un oggetto
     * <code>Prodotto</code> valorizzato solo nel campo id.
     *
     * @see it.edmaster.eLisa.Prodotto
     */
    void loadProdotto( Prodotto prodotto ) throws SQLException {
        String sql = "SELECT " +
                "NOME, " +
                "DESCRIZIONE, " +
                "PREZZO, " +
                "SCONTO, " +
                "CODICE_IVA, " +
                "CATEGORIA, " +
                "IMMAGINE, " +
                "LINK " +
            "FROM PRODOTTI WHERE ID=" + prodotto.id;
        
        Connection conn = null;
        Statement stmt = null;
        ResultSet rs = null;
        
        try {
            conn = context.getConnection();
            stmt = conn.createStatement( 
                ResultSet.TYPE_FORWARD_ONLY,
                ResultSet.CONCUR_READ_ONLY
            );
            rs = stmt.executeQuery( Logger.logSql( sql ) );
            
            Logger.log( "CatalogoDAO.loadProdotto(): id="+prodotto.id );
            
            if( rs.next() ) {
                int pos = 1;
                
                String nome = rs.getString( pos++ );
                String descrizione = rs.getString( pos++ );
                float prezzo = rs.getFloat( pos++ );
                float sconto = rs.getFloat( pos++ );
                int codiceIva = rs.getInt( pos++ );
                int categoria = rs.getInt( pos++ );
                byte[] immagine = rs.getBytes( pos++ );
                String link = rs.getString( pos++ );
                
                prodotto.setDati( nome, descrizione, prezzo, sconto, codiceIva, categoria, immagine,
                    link );
            }
        
            rs.close();
            stmt.close();
        } catch( SQLException ex ) {
            throw ex;
        } finally {
            JDBCUtils.closeAll( conn, stmt, rs );
        }
    }

}

