package it.edmaster.eLisa.catalogo;

/**
 * Definisce un tipo di codice IVA che pu essere associato ad un prodotto
 * <P>
 * La classe definisce due costanti, IVA_20 ed IVA_10 che rappresentano i due
 * valori di IVA disponibili.
 */
public class CodiceIVA {

    /**
     * IVA del 20%
     */
    public static final CodiceIVA IVA_20 = new CodiceIVA((float)0.2);
    
    /**
     * IVA del 10%
     */
    public static final CodiceIVA IVA_10 = new CodiceIVA((float)0.1);
    
    /**
     * Importo numerico che indica la percentuale dell'imposta sul valore aggiunto (IVA)
     */
    float iva;
    
    private CodiceIVA( float iva ) {
        this.iva = iva;
    }
    
    public static CodiceIVA getCodiceIVA( int codice ) {
        CodiceIVA result = null;
        
        if( codice == 1 ) {
            result = IVA_10;
        }
        if( codice == 2 ) {
            result = IVA_20;
        }
        
        return result;
    }
    
    /**
     * Ottiene il valore dell'IVA come numero decimale. Ad esempio, 10% ritorna 0.1
     */
    public float getIva() {
        return iva;
    }
    
    public String getDescrizione() {
        return (iva * 100) + "%";
    }
    
    /**
     * Ritorna l'importo dell'IVA sul prezzo del prodotto indicato
     */
    public float applicaIVA( Prodotto prodotto ) {
        return applicaIVA( prodotto.getPrezzo() );
    }

    /**
     * Ritorna l'importo dell'IVA sul prezzo indicato
     */
    public float applicaIVA( float prezzo ) {
        return prezzo + prezzo * iva;
    }
}
