package it.edmaster.eLisa.catalogo;

import java.awt.Image;
import java.net.*;
import java.sql.*;
import java.net.URL;

import it.edmaster.eLisa.*;
import it.edmaster.eLisa.util.*;

public class Prodotto {
	String id;
	
	String nome;
	String descrizione;
	
	float prezzo;
	float sconto;
	CodiceIVA iva;
	
	byte[] immagine;
    int codiceCategoriaProdotto;
	CategoriaProdotto categoria;
	URL link;
    boolean linkValido;
    
    boolean loaded = false;
    
    final int LUNGH_DESCRIZIONE_CORTA = 300;
	
    public Prodotto( String id ) {
        this( id, null, null );
    }
    
	public Prodotto( String id, String nome, String descrizione ) {
		this.id = id;
        this.nome = nome;
        this.descrizione = descrizione;
	}
    
    public String getId() {
        return id;
    }
    
    public String getNome() {
        if( nome == null ) {
            load();
        }
        return nome;
    }
    
    public String getDescrizione() {
        if( descrizione == null ) {
            load();
        }
        return descrizione;
    }
    
    /**
     * Ritorna la descrizione corta
     *
     * Questa  identificata da un numero di frasi che formino un testo da massimo 
     * LUNGH_DESCRIZIONE_CORTA caratteri.
     */
    public String getDescrizioneCorta() {
        String desc = getDescrizione();
        desc = desc.substring(0, Math.min( desc.length(), LUNGH_DESCRIZIONE_CORTA) );
        int ultimoPunto = desc.lastIndexOf(".");
        int ultimoPuntoVirgola = desc.lastIndexOf(";");
        
        int ultimoCarattere = Math.max(ultimoPunto,ultimoPuntoVirgola)+1;
        if( ultimoCarattere == 0 ) {
            ultimoCarattere = desc.length();
        }
        
        return desc.substring(0, ultimoCarattere);
    }
    
    public float getPrezzo() {
        load();
        return prezzo;
    }
    
    public float getSconto() {
        load();
        return sconto;
    }
    
	public CodiceIVA getIva() {
        load();
        return iva;
    }
	
	public byte[] getImmagine() {
        load();
        return immagine;
    }
    
	public CategoriaProdotto getCategoria() {
        load();
        return categoria;
    }
    
	public URL getLink() {
        load();
        return link;
    }
    
    public float getPrezzoFinale() {
        float prezzoFinale = iva.applicaIVA( this );
        return prezzoFinale - prezzoFinale*sconto/100;
    }
    
    void load() {
        if( !loaded ) {
            try {
                CatalogoDAO dao = DAOLocator.getInstance().getCatalogoDAO();
                dao.loadProdotto( this );
                loaded = true;
            } catch( SQLException ex ) {
                throw new RuntimeException( ex );
            }
        }
    }
    
    void setDati( String nome, String descrizione, float prezzo, float sconto, int codiceIva,
            int categoria, byte[] immagine, String link ) {
        this.nome = nome;
        this.descrizione = descrizione;
        this.prezzo = prezzo;
        this.sconto = sconto;
        this.iva = CodiceIVA.getCodiceIVA( codiceIva );
        this.immagine = immagine;
        this.codiceCategoriaProdotto = categoria;
        
        try {
            this.link = new URL( link );
            linkValido = true;
        } catch( MalformedURLException ex ) {
            linkValido = false;
        }
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer( "Prodotto=[id=" );
        sb.append( id );
        sb.append( ", nome=" );
        sb.append( nome );
        sb.append( ", descrizione=" );
        sb.append( descrizione );
        sb.append( ", prezzo=" );
        sb.append( prezzo );
        sb.append( ", sconto=" );
        sb.append( sconto );
        sb.append( ", iva=" );
        sb.append( iva );
        sb.append( ", codiceCategoriaProdotto=" );
        sb.append( codiceCategoriaProdotto );
        sb.append( ", link=" );
        sb.append( link );
        sb.append( "]" );
        
        return sb.toString();
    }
}