package it.edmaster.eLisa.ordini;

import java.sql.*;
import java.util.*;

import it.edmaster.eLisa.*;
import it.edmaster.eLisa.catalogo.*;
import it.edmaster.eLisa.carrello.*;
import it.edmaster.eLisa.common.*;
import it.edmaster.eLisa.util.*;

public class OrdiniDAO {
    ApplicationContext context;
    
    public OrdiniDAO( ApplicationContext context ) {
        this.context = context;
    }
    
    public void inoltra( Carrello carrello, String codiceUtente ) throws SQLException {
        String sql1 = "INSERT INTO ORDINI (CODUTENTE, DATAINOLTRO, IMPORTOTOTALE) VALUES (?,?,?)";
        String sql2 = "SELECT LAST_INSERT_ID() FROM ORDINI";
        String sql3 = "INSERT INTO RIGHE_ORDINI (ID_ORDINE, ID_PRODOTTO, CONTATORE, PREZZO, QUANTITA) "+
            "VALUES (?,?,?,?,?)";
        
        Connection conn = null;
        PreparedStatement stmt1 = null;
        Statement stmt2 = null;
        PreparedStatement stmt3 = null;
        ResultSet rs = null;
        
        try {
            conn = context.getConnection();
            
            stmt1 = conn.prepareStatement( Logger.logSql( sql1 ) );
            stmt2 = conn.createStatement();
            stmt3 = conn.prepareStatement( Logger.logSql( sql3 ) );
            
            int pos = 1;
            stmt1.setString( pos++, codiceUtente );
            stmt1.setDate( pos++, new java.sql.Date( new java.util.Date().getTime() ) );
            stmt1.setFloat( pos++, carrello.getTotale() );
            
            stmt1.execute();
            stmt1.close();
            
            int ordineId = 0;
            rs = stmt2.executeQuery( Logger.logSql( sql2 ) );
            if( rs.next() ) {
                ordineId = rs.getInt(1);
            } else {
                throw new SQLException("Impossibile ottenere l'ID dell'ordine appena inserito");
            }
            
            int conta = 0;
            Iterator iter = carrello.getElenco();
            while( iter.hasNext() ) {
                Prodotto prodotto = (Prodotto)iter.next();

                pos = 1;
                stmt3.setInt( pos++, ordineId );
                stmt3.setString( pos++, prodotto.getId() );
                stmt3.setInt( pos++, conta );
                stmt3.setFloat( pos++, prodotto.getPrezzoFinale() );
                stmt3.setInt( pos++, 1 );
                stmt3.execute();
                
                conta++;
            }
            stmt2.close();
            
        } catch( SQLException ex ) {
            throw ex;
        } finally {
            JDBCUtils.close( rs );
            JDBCUtils.close( stmt1 );
            JDBCUtils.close( stmt2 );
            JDBCUtils.close( stmt3 );
            JDBCUtils.close( conn );
        }                
    }
    
    Pagina getElenco( String codiceUtente, boolean evaso, int inizio, int quanti ) 
            throws SQLException {
                
        Pagina ritorno = null;
        String sql = "SELECT ID, CODUTENTE, DATAINOLTRO, IMPORTOTOTALE, EVASO FROM ORDINI WHERE " +
            "EVASO=" + (evaso ? 1 : 0 );
        
        if( codiceUtente != null ) {
            sql += " AND CODUTENTE='" + codiceUtente + "'";
        }
        
        sql += " ORDER BY DATAINOLTRO DESC";
        
        Connection conn = null;
        Statement stmt = null;
        ResultSet rs = null;
        
        try {
            conn = context.getConnection();
            stmt = conn.createStatement( 
                ResultSet.TYPE_SCROLL_INSENSITIVE,
                ResultSet.CONCUR_READ_ONLY
            );
            rs = stmt.executeQuery( Logger.logSql( sql ) );
            
            Logger.log( "OrdiniDAO.loadElenco(): inizio="+inizio + " absolute="+rs.absolute(inizio+1) );
            
            if (inizio >= 0 && rs.absolute(inizio+1)) {
                boolean hasNext = false;
                List elenco = new LinkedList();
                
                do {
                    int pos = 1;
                    
                    Ordine ordine = null;
                    String id = rs.getString( pos++ );
                    
                    if( !CacheManager.getCacheOrdini().contains( id ) ) {
                        String codUtente = rs.getString( pos++ );
                        java.sql.Date data = rs.getDate( pos++ );
                        float importo = rs.getFloat( pos++ );
                        //boolean evaso = rs.getBoolean( pos++ );
                        
                        ordine = new Ordine( id, codUtente, data, importo, evaso );
                        
                        //Salva l'ordine nella cache
                        CacheManager.getCacheOrdini().add( id, ordine );
                    } else {
                        ordine = (Ordine)CacheManager.getCacheOrdini().get( id );
                    }
                    elenco.add( ordine );
                    
                    Logger.log( "OrdiniDAO.loadElenco(): ordine="+ordine );
                    
                } while ((hasNext = rs.next()) && (--quanti > 0));
                
                ritorno = new Pagina(elenco, inizio, hasNext);
            } else {
                ritorno = Pagina.VUOTA;
            }
        
            rs.close();
            stmt.close();
        } catch( SQLException ex ) {
            throw ex;
        } finally {
            JDBCUtils.closeAll( conn, stmt, rs );
        }
        
        return ritorno;
    }
    
}

