package it.edmaster.eLisa.utenti;

import it.edmaster.eLisa.*;

import java.util.*;

public class ElencoUtentiHelper {
    
    ElencoUtenti elencoUtenti = new ElencoUtenti();
    
    public boolean autentica( String codiceUtente, String password ) throws CaricamentoUtentiException {
        return elencoUtenti.autentica( codiceUtente, password );
    }
    
    public List getElenco() throws CaricamentoUtentiException {
        return elencoUtenti.getElenco();
    }
    
    public Utente getUtente( String codiceUtente ) throws CaricamentoUtentiException {
        return elencoUtenti.get( codiceUtente );
    }

    public void inserisci( String codiceUtente, String password1, String password2, String nome,
            String cognome) throws InserimentoUtentiException, CaricamentoUtentiException {
                
        StringBuffer messaggioErrore = new StringBuffer();
        InserimentoUtentiException ex = new InserimentoUtentiException();
        
        boolean passwordCorta = false;
        boolean datiMancanti = false;
        boolean passwordErrate = false;
        boolean utenteInUso = false;
        
        if( password1 == null || password1.trim().length() == 0 ) {
            ex.addCampo( InserimentoUtentiException.PASSWORD1 );
            datiMancanti = true;
        } else {
            passwordCorta = password1.length() < 6;
        }
        
        if( password2 == null || password1.trim().length() == 0 ) {
            ex.addCampo( InserimentoUtentiException.PASSWORD2 );
            datiMancanti = true;
        } else {
            passwordCorta |= password2.length() < 6;
        }
        
        if( codiceUtente == null || codiceUtente.trim().length() == 0 ) {
            ex.addCampo( InserimentoUtentiException.CODICE_UTENTE );
            datiMancanti = true;
        } else {
            Utente utente = elencoUtenti.get( codiceUtente );
            if( !(utente instanceof UtenteNonEsistente) ) {
                ex.addCampo( InserimentoUtentiException.CODICE_UTENTE );
                messaggioErrore.append( "Il codice utente  gi in uso" );
                utenteInUso = true;
            }
        }
        
        if( nome == null || nome.trim().length() == 0 ) {
            ex.addCampo( InserimentoUtentiException.NOME );
            datiMancanti = true;
        }
        if( cognome == null || cognome.trim().length() == 0 ) {
            ex.addCampo( InserimentoUtentiException.COGNOME );
            datiMancanti = true;
        }
        
        if( passwordCorta ) {
            if( messaggioErrore.length() > 0 ) {
                messaggioErrore.append( ", " );
            }
            messaggioErrore.append("le parole chiave devono essere lunghe almeno sei caratteri" );
        }
        
        if( datiMancanti ) {
            if( messaggioErrore.length() > 0 ) {
                messaggioErrore.append( ", " );
            }
            messaggioErrore.append("alcuni dati sono mancanti" );
        }
        
        //Controllo la corrispondenza delle password solo se hanno passato i controlli
        //precedenti, e cio se sono presenti e pi lunghe di 6 caratteri
        if( password1 != null ) {
            if( !password1.equals( password2 ) ) {
                if( messaggioErrore.length() > 0 ) {
                    messaggioErrore.append( ", " );
                }
                messaggioErrore.append("le parole chiave non coincidono" );
                
                ex.addCampo( InserimentoUtentiException.PASSWORD1 );
                ex.addCampo( InserimentoUtentiException.PASSWORD2 );
                passwordErrate = true;
            }
        }

        if( passwordCorta || datiMancanti || passwordErrate || utenteInUso ) {
            ex.setMessage( messaggioErrore.toString() );
            throw ex;
        } else {
            elencoUtenti.inserisci( codiceUtente, password1, nome, cognome );
        }
    }
    
    public String cercaPassword( String codiceUtente, String nome, String cognome ) 
            throws CaricamentoUtentiException {

        if( codiceUtente != null && codiceUtente.length()>0 ) {
            Utente utente = elencoUtenti.get( codiceUtente );
            if( utente.getNome().equals( nome ) && utente.getCognome().equals( cognome ) ) {
                return utente.getPassword();
            }
        }

        return null;
    }
}
