package it.edmaster.eLisa.utenti;

import java.sql.*;
import java.util.*;

import it.edmaster.eLisa.*;
import it.edmaster.eLisa.common.*;
import it.edmaster.eLisa.util.*;

public class UtentiDAO {
    ApplicationContext context;

    /** @link dependency */
    /*#CaricamentoUtentiException lnkCaricamentoUtentiException;*/

    /** @link dependency */
    /*#InserimentoUtentiException lnkInserimentoUtentiException;*/
    
    public UtentiDAO( ApplicationContext context ) {
        this.context = context;
    }
    
    public boolean autentica( String codiceUtente, String password ) throws SQLException {
        if( !CacheManager.getCacheUtenti().contains( codiceUtente ) ) {
            loadUtente( codiceUtente );
        }
        Utente utente = (Utente)CacheManager.getCacheUtenti().get( codiceUtente );
        
        //Verifica che l'utente non si sia registrato nel sistema dopo aver provato una
        //prima connessione
        if( utente instanceof UtenteNonEsistente ) {
            loadUtente( codiceUtente );
            utente = (Utente)CacheManager.getCacheUtenti().get( codiceUtente );
        }
        
        return utente.autentica( password );
    }
    
    /**
     * Ritorna l'elenco degli utenti presenti nel sistema, rinfrescandone tutti gli attributi
     * Operazione pesante, ma necessaria per avere, da parte dell'amministratore del sistema,
     * i dati sempre aggiornati degli utenti del sistema che possono autonomamente, in qualsiasi
     * momento, iscriversi e cambiare i propri dati (anche se quest'ultima funzionalit non 
     * stata implementata nella versione corrente del sistema).
     */
    public List getElenco() throws SQLException {
        String sql = "SELECT ID, CODUTENTE, PASSWORD, NOME, COGNOME, LIVELLO FROM UTENTI ORDER BY COGNOME, NOME";
        
        List elenco = new LinkedList();
        
        Connection conn = null;
        Statement stmt = null;
        ResultSet rs = null;
        
        try {
            conn = context.getConnection();
            stmt = conn.createStatement( 
                ResultSet.TYPE_FORWARD_ONLY,
                ResultSet.CONCUR_READ_ONLY
            );
            rs = stmt.executeQuery( Logger.logSql( sql ) );
            
            while( rs.next() ) {
                int pos = 1;
                
                Utente utente = new Utente( 
                    rs.getInt( pos++ ),
                    rs.getString( pos++ ),
                    rs.getString( pos++ ),
                    rs.getString( pos++ ),
                    rs.getString( pos++ ),
                    rs.getInt( pos++ )
                );
                
                Logger.log( "UtentiDAO.loadUtente(): utente="+utente );
                
                elenco.add( utente );
                CacheManager.getCacheUtenti().add( utente.getCodiceUtente(), utente );
            }
    
            rs.close();
            stmt.close();
        } catch( SQLException ex ) {
            throw ex;
        } finally {
            JDBCUtils.closeAll( conn, stmt, rs );
        }
        
        return elenco;
    }
    
    public Utente get( String codiceUtente ) throws SQLException {
        if( !CacheManager.getCacheUtenti().contains( codiceUtente ) ) {
            loadUtente( codiceUtente );
        }
        Utente utente = (Utente)CacheManager.getCacheUtenti().get( codiceUtente );
        return utente;
    }
    
    public void loadUtente( String codiceUtente ) throws SQLException {
        String sql = "SELECT ID, PASSWORD, NOME, COGNOME, LIVELLO FROM UTENTI WHERE CODUTENTE='"
            + codiceUtente + "'";
        
        Connection conn = null;
        Statement stmt = null;
        ResultSet rs = null;
        
        try {
            conn = context.getConnection();
            stmt = conn.createStatement( 
                ResultSet.TYPE_FORWARD_ONLY,
                ResultSet.CONCUR_READ_ONLY
            );
            rs = stmt.executeQuery( Logger.logSql( sql ) );
            
            if( rs.next() ) {
                int pos = 1;
                
                Utente utente = new Utente( 
                    rs.getInt( pos++ ),
                    codiceUtente,
                    rs.getString( pos++ ),
                    rs.getString( pos++ ),
                    rs.getString( pos++ ),
                    rs.getInt( pos++ )
                );
                
                Logger.log( "UtentiDAO.loadUtente(): utente="+utente );
                
                CacheManager.getCacheUtenti().add( codiceUtente, utente );
            } else {
                CacheManager.getCacheUtenti().add( codiceUtente, Utente.getUtenteNonEsistente() );
            }
    
            rs.close();
            stmt.close();
        } catch( SQLException ex ) {
            throw ex;
        } finally {
            JDBCUtils.closeAll( conn, stmt, rs );
        }
    }
    
    public void inserisci( String codiceUtente, String password, String nome,
            String cognome) throws SQLException {
                
        String sql = "INSERT INTO UTENTI (CODUTENTE, PASSWORD, NOME, COGNOME, LIVELLO) VALUES (" +
            "'" + codiceUtente + "', " +
            "'" + password + "', " +
            "'" + nome + "', " +
            "'" + cognome + "', " +
            "0" +
            ")";
        
        Connection conn = null;
        Statement stmt = null;
        
        try {
            conn = context.getConnection();
            
            stmt = conn.createStatement();
            stmt.executeUpdate( Logger.logSql( sql ) );
            stmt.close();
            
            //Non possiamo creare direttamente un utente ed inserirlo in cache in quanto non 
            //noto l'ID. Un approccio alternativo  quello di ottenerlo dal database.
            loadUtente( codiceUtente );
            
        } catch( SQLException ex ) {
            throw ex;
        } finally {
            JDBCUtils.closeAll( conn, stmt );
        }                
    }
    
}

