package it.edmaster.eLisa.util;

import java.sql.*;
import java.util.*;

import javax.naming.*;
import javax.sql.*;

/**
 * Classe Singleton utilizzata per individuare, tramite JNDI, risorse esterne necessarie all'applicazione,
 * come il DataSource per accedere al database.
 * <P>
 * @see JNDI http://java.sun.com/products/jndi
 */
public class ServiceLocator {

    /**
     * DataSource ottenuto dal Web Container tramite JNDI
     */
    DataSource dataSource;
    
    /**
     * Instanza del Singleton
     */
    static ServiceLocator instance = null;
    
    /**
     * Nome del riferimento JNDI al database
     */
    static final String DATASOURCE_NAME = "jdbc/eLisa";
    
    private ServiceLocator() {
        locateDataSourcePool();
    }
    
    /**
     * Ritorna un oggetto <code>ServiceLocator</code>
     */
    public static ServiceLocator getInstance() {
        if( instance == null ) {
            instance = new ServiceLocator();
        }
        return instance;
    }
    
    /**
     * Ritorna il Data Source dell'applicazione
     */
    public DataSource getDataSource() {
        return dataSource;
    }
    
    /**
     * Individua il Data Source individuato dal nome indicato dalla costante <code>DATASOURCE_NAME</code>
     */
    void locateDataSourcePool() {
        try {
            Context initContext = new InitialContext();
            Context envContext  = (Context)initContext.lookup("java:comp/env");
            Object o = envContext.lookup(DATASOURCE_NAME);
            dataSource = (javax.sql.DataSource)o;
        } catch( Exception ex ) {
            throw new RuntimeException( ex );
        }
    }
    
}
