package cap04;

import java.sql.*;

public class JDBCTest {
    
    protected Connection conn;
    
    public JDBCTest() {
        try {
            connect();
            
            dropTable();
            createTable();
            insertValues1();
            deleteValues();
            insertValues2();
            getValues();
            
            getAndChangeValues();
            getValues();
            
            conn.close();
            
        } catch( Exception ex ) {
            ex.printStackTrace();
        }
    }
    
    protected void connect() throws ClassNotFoundException, InstantiationException, 
            IllegalAccessException, SQLException {
        Class.forName("com.mysql.jdbc.Driver").newInstance();
        conn = DriverManager.getConnection("jdbc:mysql:///prova?user=max&password=max");
    }
    
    protected void dropTable() {
        try {
            String sql = "DROP TABLE os";
            System.out.println( sql );
            
            Statement stmt = conn.createStatement();
            stmt.execute( sql );
            stmt.close();
        } catch( SQLException ex ) {
            System.err.println("La tabella 'os' non esiste");
            ex.printStackTrace();
        }
    }
    
    protected void createTable() throws SQLException {
        String sql = "CREATE TABLE os ("+
            "id int(11) NOT NULL auto_increment,"+
            "nome varchar(100) default '',"+
            "PRIMARY KEY  (id)"+
            ") TYPE=MyISAM ";
        System.out.println( sql );

        Statement stmt = conn.createStatement();
        stmt.execute( sql );
        stmt.close();
    }
    
    protected void insertValues1() throws SQLException {
        String sql = "INSERT INTO os (nome) VALUES ('Windows')";
        System.out.println( sql );
        
        Statement stmt = conn.createStatement();
        boolean result = stmt.execute(sql);

        System.out.println("insertValues1(): result="+result);
        System.out.println("insertValues1(): getUpdateCount()="+stmt.getUpdateCount());
        
        stmt.close();
    }
    
    protected void deleteValues() throws SQLException {
        String sql = "DELETE FROM os WHERE nome='Windows'";
        System.out.println( sql );
        
        Statement stmt = conn.createStatement();
        boolean result = stmt.execute(sql);

        System.out.println("deleteValues(): result="+result);
        System.out.println("deleteValues(): getUpdateCount()="+stmt.getUpdateCount());
        
        stmt.close();
    }
    
    protected void insertValues2() throws SQLException {
        String[] valori = {"Windows", "Unix", "Linux", "Mac OS X", "Free BSD", "OS/2", "Amiga"};
        
        String sql = "INSERT INTO os (nome) VALUES (?)";
        System.out.println( sql );
        
        PreparedStatement stmt = conn.prepareStatement(sql);
        
        for( int i=0; i<valori.length; i++ ) {
            stmt.setString(1, valori[i]);
            int count = stmt.executeUpdate();
            
            System.out.println("Inserito " + valori[i] + " count="+count+" ("+i+")");
        }
        
        stmt.close();
    }
    
    protected void getValues() throws SQLException {
        String sql = "SELECT id, nome FROM os ORDER BY nome";
        System.out.println( sql );
        
        Statement stmt = conn.createStatement();
        ResultSet rs = stmt.executeQuery(sql);
        
        while( rs.next() ) {
            System.out.print( rs.getString(1) );
            System.out.print( " - " );
            System.out.println( rs.getString(2) );
        }
        
        rs.close();
        stmt.close();
    }
    
    protected void getAndChangeValues() throws SQLException {
        String sql = "SELECT id, nome FROM os ORDER BY nome";
        System.out.println( sql );
        
        Statement stmt = conn.createStatement(
            ResultSet.TYPE_FORWARD_ONLY,
            ResultSet.CONCUR_UPDATABLE);
        ResultSet rs = stmt.executeQuery(sql);
        
        while( rs.next() ) {
            String nome = rs.getString(2);
            
            if( nome.equals("Windows") ) {
                rs.updateString(2, "Windows XP");
                System.out.println("Ho cambiato Windows con Windows XP");
                System.out.println("Ora nome vale (1) " + rs.getString(2));
                
                rs.updateRow();
                System.out.println("Ora nome vale (2) " + rs.getString(2));
            }
            
        }
        
        rs.close();
        stmt.close();
    }
    
    public static void main( String[] args ) {
        new JDBCTest();
    }
}
