static char rcs_id [] = "$Id: crobcons32.c,v 1.6 1998/06/08 13:50:32 Misha Exp $";
/* ========================================================================= *
   Description: console interface to a 'crobots32' program

   Usage:  crobcons32 [flags] <crobot1> [ . . . <crobot_n>]

   Author: Mikhail Verkhovski,
           via Singen 26, Pomezia 00040, Italia
           misha@hite.it

   Notes:  'crobcons32' is a simple console interface to a 'crobots32' crobots
           managing program. It's aim is to illustrate the connectivity
           mechanism and encourage other programmers to create more
           beautiful windows crobots applications.

           'crobcons32' adds the -s flag to a command line and spawns 
           'crobots32' in supervised mode. It uses pipe to read the
           crobots32's output and update the screen.

           I use threads to draw the explosions and CRITICAL_SECTION global
           variable Syng to synchronize them.

           To compile a 'crobcons32' program I use the
           Microsoft (R) 32-bit Optimizing compiler Version 11 supplied
           with Microsoft (R) Visual C++ 5.0 package.

   Compilation:
           cl /MT crobcons32.c

   License agreement:

           This code have only illustrative propose. You may make copies
           of this code and it's parts, give it to your friends, or include
           it in the library of a non-profit computer club.

           I expressly forbid any for-profit venture from selling this
           software.

   DISCLAIMER OF WARRANTY:

           THIS SOFTWARE ARE PROVIDED "AS IS" WITHOUT WARRANTY OF
           AND KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED
           TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS OF
           PURPOSE. THE USER OF THIS SOFTWARE ASSUME ALL RISKS.

 * ========================================================================= */
#include <windows.h>
#include <process.h>                   /* _beginthread, _endthread           */
#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>
                                       /* max value for crobot coordinates   */
#define MAX_POSITION ((double)RAND_MAX * 1000.0 / ((double)RAND_MAX+1.0))

/* ========================================================================= *
   Function prototypes
 * ========================================================================= */
void Explosion (void *coord);
void Collision (void *coord);
void DrawCross (COORD center, WORD att, int mode, int ind);
void WriteAttribute (COORD coord, WORD att, int mode);
double GetValue (char *prefix, char *buffer);

/* ========================================================================= *
   Global variables
 * ========================================================================= */
CRITICAL_SECTION Sync;                 /* For synchronization                */
HANDLE StdOut;                         /* Handle for console window          */
CONSOLE_SCREEN_BUFFER_INFO Csbi;       /* Console information structure      */
unsigned long *ThId;                   /* Thread Ids                         */
int MaxX, MaxY;                        /* Screen size                        */
char **Out = NULL;                     /* Remember output                    */
int NumOut = 0;

                                       /* Foreground screen attributes       */
WORD Normal = FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE;
WORD Bright = FOREGROUND_RED | FOREGROUND_GREEN | FOREGROUND_BLUE |
                                                         FOREGROUND_INTENSITY;

/* ========================================================================= *
   main
   -------------------------------------------------------------------------
   Return: void
 * ========================================================================= */
void main (int argc, char *argv[])
{
  FILE *pipe;
  int i, num;
  char *c;
  double x, y;
  char ch, blank = ' ';
  COORD coord[100], *pcoord;
  DWORD result;
  char buffer [MAX_PATH+1];
                                       /* init critical section              */
  InitializeCriticalSection (&Sync);
                                       /* prepare command line to spawn      */
  strcat (buffer, "crobots32 ");
  for (i=1; i<argc; i++) {
    strcat (buffer, argv[i]);
    strcat (buffer, " ");
  }
  strcat (buffer, "-s");
                                       /* start crobots32                    */
  pipe = _popen (buffer, "rt");
  if (pipe == NULL) {
    printf ("Error spawn crobots32: %s\n", strerror (errno));
    exit (-1);
  }
                                       /* get consol handle                  */
  StdOut = GetStdHandle (STD_OUTPUT_HANDLE);
  if (StdOut == INVALID_HANDLE_VALUE) {
    printf ("Error obtaining consol handler: %d\n", GetLastError ());
    exit (-1);
  }
                                       /* get display screen's text
                                          row and column information         */
  GetConsoleScreenBufferInfo (StdOut, &Csbi);

                                       /* do not use last row e last column  */
  MaxX = Csbi.dwSize.X - 1;
  MaxY = Csbi.dwSize.Y - 1;
                                       /* allocate thread id buffer          */
  ThId = (unsigned long *)calloc (MaxX * MaxY, sizeof (unsigned long)); 
  if (ThId == NULL) {
    printf ("Error allocating memory: %s\n", strerror (errno));
    exit (-1);
  }
                                       /* clean the screen                   */
  coord[0].X = 0;
  coord[0].Y = 0;
  FillConsoleOutputCharacter (StdOut, blank,
                             Csbi.dwSize.X * Csbi.dwSize.Y, coord[0], &result);
  FillConsoleOutputAttribute (StdOut, Bright,
                             Csbi.dwSize.X * Csbi.dwSize.Y, coord[0], &result);

                                       /* put the cursor on last row         */
  coord[0].X = 0;
  coord[0].Y = Csbi.dwSize.Y - 1;
  SetConsoleCursorPosition (StdOut, coord[0]);

                                       /* start main cycle                   */
  while (!feof (pipe)) {
    if (fgets (buffer, MAX_PATH, pipe) != NULL) {

      switch (buffer[0]) {
                                       /* memo for output after the match    */
      case 'O':
        Out = realloc (Out, (NumOut+1) * sizeof (char *));
        if (Out == NULL) {
          printf ("Error allocating memory: %s\n", strerror (errno));
          exit (-1);
        }
        Out [NumOut] = strdup (buffer+2);
        if (Out[NumOut] == NULL) {
          printf ("Error allocating memory: %s\n", strerror (errno));
          exit (-1);
        }
        NumOut++;
        break;
                                       /* crobot position                    */
      case 'P':
                                       /* crobot's number e coordinates      */
        c = strchr (buffer+2, ':');
        do c--; while (isdigit (*c));
        num = atoi (c+1);
        x = GetValue ("x=", buffer);
        y = GetValue ("y=", buffer);
        ch = '0' + num;
                                       /* draw the crobot                    */
        WriteConsoleOutputCharacter (StdOut, &blank, 1, coord[num], &result);
        coord[num].X = x * MaxX / 1000.0;
        coord[num].Y = (MAX_POSITION - y) * MaxY / 1000.0;
        WriteConsoleOutputCharacter (StdOut, &ch, 1, coord[num], &result);
        break;
                                       /* explosion                          */
      case 'X':
        x = GetValue ("x=", buffer);
        y = GetValue ("y=", buffer);
                                       /* allocate the COORD structure       */
        pcoord = calloc (1, sizeof (COORD));
        pcoord->X = x * MaxX / 1000.0;
        pcoord->Y = (MAX_POSITION - y) * MaxY / 1000.0;

                                       /* start explosion thread             */
        _beginthread (Explosion, 0, pcoord);
        break;
                                       /* collision                          */
      case 'C':
        x = GetValue ("x=", buffer);
        y = GetValue ("y=", buffer);
                                       /* allocate the COORD structure       */
        pcoord = calloc (1, sizeof (COORD));
        pcoord->X = x * MaxX / 1000.0;
        pcoord->Y = (MAX_POSITION - y) * MaxY / 1000.0;

                                       /* start collision thread             */
        _beginthread (Collision, 0, pcoord);
        break;

                                       /* crobot died                        */
      case 'D':
        c = strchr (buffer+2, ':');
        do c--; while (isdigit (*c));
        num = atoi (c+1);
        
        WriteConsoleOutputCharacter (StdOut, &blank, 1, coord[num], &result);
        break;
      }
    }
  }
  coord[0].X = 0;
  coord[0].Y = Csbi.dwSize.Y - 1;
  WriteConsoleOutputCharacter (StdOut, "Press any key . . .", 19,
                                                           coord[0], &result);
  getch ();
  coord[0].X = 0;
  coord[0].Y = 0;
  FillConsoleOutputCharacter (StdOut, blank,
                            Csbi.dwSize.X * Csbi.dwSize.Y, coord[0], &result);
  FillConsoleOutputAttribute (StdOut, Normal,
                            Csbi.dwSize.X * Csbi.dwSize.Y, coord[0], &result);
                                       /* print the match results            */
  SetConsoleCursorPosition (StdOut, coord[0]);
  for (i=0; i<NumOut; i++)
    printf (Out[i]);
}

/* ========================================================================= *
   Explosion
   -------------------------------------------------------------------------
   Draw explosion in real time
   Return: void
 * ========================================================================= */
void Explosion (void *param)
{
                                       /* background attributes              */
  WORD center = BACKGROUND_GREEN | BACKGROUND_RED | BACKGROUND_INTENSITY;
  WORD row = BACKGROUND_RED | BACKGROUND_BLUE | BACKGROUND_INTENSITY;
  WORD angle = BACKGROUND_RED | BACKGROUND_INTENSITY;

  COORD coord;

  memcpy (&coord, param, sizeof (COORD));
  free (param);

  WriteAttribute (coord, center, 0);

  Sleep (200);
  DrawCross (coord, row, 0, 0);

  Sleep (200);
  DrawCross (coord, angle, 0, 1);

  Sleep (800);

  WriteAttribute (coord, Bright, 1);

  Sleep (200);
  DrawCross (coord, Bright, 1, 0);

  Sleep (200);
  DrawCross (coord, Bright, 1, 1);
}

/* ========================================================================= *
   Collision
   -------------------------------------------------------------------------
   Draw collision
   Return: void
 * ========================================================================= */
void Collision (void *param)
{
  WORD center = BACKGROUND_GREEN | BACKGROUND_INTENSITY;

  COORD coord;

  memcpy (&coord, param, sizeof (COORD));
  free (param);

  WriteAttribute (coord, center, 0);

  Sleep (1000);

  WriteAttribute (coord, Bright, 1);
}

/* ========================================================================= *
   DrawCross
   -------------------------------------------------------------------------
   Draw a cross on a screem
   Return: void
 * ========================================================================= */
void DrawCross (COORD center, WORD att, int mode, int ind)
{
  COORD coord;
  int i;

static const COORD shift[2][4] = {
  {{0, -1}, {0, 1}, {-1, 0}, {1, 0}},
  {{-1, -1}, {1, 1}, {-1, 1}, {1, -1}}
};

  for (i=0; i<4; i++) {
    coord.X = center.X + shift[ind][i].X;
    coord.Y = center.Y + shift[ind][i].Y;
    if (coord.X >= 0 && coord.Y >= 0 && coord.X < MaxX && coord.Y < MaxY)
      WriteAttribute (coord, att, mode);
  }
}

/* ========================================================================= *
   WriteAttribute
   -------------------------------------------------------------------------
   Change an attribute of a one cell on a screen.
     mode = 0: set a cell attribute and change a cell thread id to our id
     mode = 1: set a cell attribute only if it's id is our id
   Return: void
 * ========================================================================= */
void WriteAttribute (COORD coord, WORD att, int mode)
{
  DWORD result;
  int ind = coord.Y * MaxX + coord.X;

  EnterCriticalSection (&Sync);
  if (mode == 1) {
    if (ThId [ind] == _threadid) {
      WriteConsoleOutputAttribute (StdOut, &att, 1, coord, &result);
      ThId [ind] = 0;
    }
  } else {
    WriteConsoleOutputAttribute (StdOut, &att, 1, coord, &result);
    ThId [ind] = _threadid;
  }
  LeaveCriticalSection (&Sync);
}

/* ========================================================================= *
   GetValue
   -------------------------------------------------------------------------
   Return a value of a parameter in a string after a prefix
   Return: double
 * ========================================================================= */
double GetValue (char *prefix, char *buffer)
{
  return (atof (strstr (buffer, prefix)+strlen (prefix)));
}

