/***************************************************************************
                             token.h  -  Tokenizer
                             -----------------------------------
    begin                : thu jun 6 11:00:36 CET 2000
    copyright            : (C) 2000 by Marco Giovannini
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TOKEN_H
#define TOKEN_H

#include "ppc.h"
#include "slist.h"

class Token
{
public:
    Token(char* str=NULL)     { setString(str); }
    char* getString(void)     { return string; }
    void setString(char* str) { string=str; }

private:
    char* string;
};

class TokenList
{
public:
    TokenList(void)        { prec=next=NULL; curr=NULL; }
    void reset(void);

    void add(Token* e)     { curr?next->enqueue(e):setCurr(e); }
    void insert(Token* e)  { curr?next->push(e):setCurr(e); }

    void add(char* t)      { add(new Token(t)); }
    void insert(char* t)   { insert(new Token(t)); }

    void remove(void);

    void goFirst(void)     { _goFirst(); setCurr(next->pop()); }
    void goNext(void) 
        { if (curr) prec->enqueue(curr); setCurr(next->pop()); }
    void goLast(void)      { while (!next->empty()) goNext(); }

    Token* getCurr(void)   { return curr; }
    void setCurr(Token* c) { curr=c; }

private:
    void _goFirst(void);

    SList<Token>* prec;
    Token*        curr;
    SList<Token>* next;
};

class Tokenizer
{
public:
    void tokenize(char* buffer);

    char* getToken(void) { return tl.getCurr()?tl.getCurr()->getString():NULL; }
    void putToken(char* str) { tl.insert(new Token(str)); }
    void removeToken(void) { tl.remove(); }
    void changeToken(char* str);

    char* nextToken(void) { tl.goNext(); return getToken(); }
    char* trim(void);
    char* trimNext(void)  { nextToken(); return trim(); }

    void firstToken(void) { tl.goFirst(); }
    void lastToken(void)  { tl.goLast(); }

    void printAll(FILE* dst=stdout);

private:
    int is_valid(char ch);

    int  t_end;
    char tokens[BUFSIZE];
    TokenList tl;
};

#endif
