/***************************************************************************
                             token.cc  -  Tokenizer
                             -----------------------------------
    begin                : thu jun 6 11:00:36 CET 2000
    copyright            : (C) 2000 by Marco Giovannini
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include "h/ppc.h"
#include "h/slist.h"
#include "h/token.h"

void TokenList::reset(void)
{
    if (prec) delete prec;
    prec=new SList<Token>;
    if (next) delete next;
    next=new SList<Token>;
    if (curr) delete curr;
    curr=NULL;
};

void TokenList::_goFirst(void)
{
    Token* tmp=prec->pop();
    if (tmp) _goFirst(); else tmp=getCurr();
    if (tmp) next->push(tmp);
}

void TokenList::remove(void)
{
    if (Token* tmp=getCurr()) {
        goNext();
        prec->erase(tmp);
    }
}

int Tokenizer::is_valid(char ch)
{
    switch (ch) {
        case ',': case ';': case '+': case '*': case '!': case '/':
        case '(': case ')': case '=': case '{': case '}': case '~':
        case '>': case '<': case '?': case ':': case ' ': case '\t':
        case '\n': return FALSE;
        default: return TRUE;
    }
}

char* Tokenizer::trim(void)
{
    char* tok=getToken();
    while (tok && (*tok=='\t' || *tok==' ' || *tok=='\n'))
        tok=nextToken();
    return tok;
}

void Tokenizer::printAll(FILE* dst)
{
    firstToken();
    char* tok=getToken();
    while (tok) {
        fprintf(dst,"%s",tok);
        tok=nextToken();
    }
}

void Tokenizer::tokenize(char* buffer)
{
    tl.reset();
    if (buffer && strlen(buffer)>0) {
        int sp_ch=TRUE;
        t_end=0;
        do {
            if (is_valid(*buffer)) {
                if (sp_ch) {
                    tl.add(new Token(&tokens[t_end]));
                    sp_ch=FALSE;
                }
                tokens[t_end++]=*buffer;
            } else {
                if (!sp_ch) tokens[t_end++]='\0';
                tl.add(new Token(&tokens[t_end]));
                tokens[t_end++]=*buffer;
                tokens[t_end++]='\0';
                sp_ch=TRUE;
            }
        } while (*(++buffer)!='\0');
        tokens[t_end++]='\0';
    }
}

void Tokenizer::changeToken(char* str)
{
    if (tl.getCurr()) {
        tl.getCurr()->setString(&tokens[t_end]);
        while (tokens[t_end++]=*str++) ;
    }
}

